function CAMOEA(Global)
% A Clustering Based Adaptive Evolutionary Algorithm for Multi-Objective
% Optimization with Irregular Pareto Fronts

    Population = Global.Initialization();

    while Global.NotTermination(Population)
        % Generate offspring randomly
        MatingPool = randperm(Global.N);
        Offspring  = Global.Variation(Population(MatingPool));
        
        % Elitism strategy
        UniPop = [Population,Offspring];
        PopObj = UniPop.objs;
        [FrontNo,MaxFNo] = NDSort(PopObj,Global.N);
        
        % The number of individuals to be selected in the last
        % non-dominated front
        K = Global.N-sum(FrontNo<MaxFNo);
        
        if K~= 0
            % Normalization
            pareto_population = find(FrontNo<MaxFNo);
            last_population = find(FrontNo == MaxFNo);
            Zmin = min(PopObj(FrontNo == MaxFNo,:));
            Zmax = max(PopObj(FrontNo == MaxFNo,:));
            S = sum(FrontNo == MaxFNo);
            MaxFnorm = (PopObj(last_population,:)-repmat(Zmin,S,1))./repmat(Zmax-Zmin,S,1);
            
            % Clustering-based reference points generation
            [Ref] = Reference_Generation( MaxFnorm,Global.M,K);
            
            % Clustering-based environmental selection
            [reference_population] = Reference_Point_Selection(MaxFnorm,last_population,Ref,K,Global.M);
            
        else
            pareto_population = find(FrontNo<=MaxFNo);
            reference_population = [];
            
        end

        Population = UniPop([pareto_population,reference_population]);
                
    end
end








